Package: panelView
Type: Package
Title: Visualizing Panel Data
Version: 1.3.0
Date: 2026-04-08
Authors@R: 
  c(person("Yiqing", "Xu", ,"yiqingxu@stanford.edu", role = c("aut", "cre"), 
  comment = c(ORCID = "0000-0003-2041-6671")),
  person("Hongyu", "Mou", , "hongyumou@g.ucla.edu", role = c("aut")), 
  person("Licheng", "Liu", ,"liulch@mit.edu", role = c("aut")))
Maintainer: Yiqing Xu <yiqingxu@stanford.edu>
Description: Visualizes panel data. It has four main functionalities: (1) it plots the treatment status and missing values in a panel dataset; (2) it visualizes the temporal dynamics of a main variable of interest; (3) it depicts the bivariate relationships between a treatment variable and an outcome variable either by unit or in aggregate; (4) it displays the network structure of multi-way fixed effects as a k-partite graph, identifying connected components, singletons, and duplicate observations. For details, see <doi:10.18637/jss.v107.i07>.
URL: https://yiqingxu.org/packages/panelview/,
        https://github.com/xuyiqing/panelView
BugReports: https://github.com/xuyiqing/panelView/issues
License: MIT + file LICENSE
Imports: ggplot2 (>= 3.4.0), gridExtra, grid, dplyr (>= 1.0.0)
Depends: R (>= 2.10)
Encoding: UTF-8
Suggests: testthat (>= 3.0.0), igraph
RoxygenNote: 7.3.2
LazyData: true
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-08 22:23:02 UTC; yiqingxu
Author: Yiqing Xu [aut, cre] (ORCID: <https://orcid.org/0000-0003-2041-6671>),
  Hongyu Mou [aut],
  Licheng Liu [aut]
Repository: CRAN
Date/Publication: 2026-04-10 08:40:07 UTC
Built: R 4.6.0; ; 2026-04-26 13:22:44 UTC; unix
