.onAttach <-
function (libname, pkgname){
   # echo output to screen
   packageStartupMessage("## See https://yiqingxu.org/packages/panelview/ for more info.\n## Report bugs -> yiqingxu@stanford.edu.\n")
}
