\name{turnout}
\docType{data}
\alias{turnout}
\title{Voter turnout data.}
\description{State-level voter turnout data.}
\format{dataframe}
\keyword{datasets}
\references{
  Melanie Jean Springer (2014). How the States Shaped the Nation: American Electoral Institutions and Voter Turnout, 1920-2000. University of Chicago Press.

  Yiqing Xu (2017). "Generalized Synthetic Control Method: Causal Inference with Interactive Fixed Effects Models." Political Analysis, 25(1), pp. 57--76.  
}

